@@ -17,34 +17,42 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#ifdef HAVE_LOCALE_H
-#include <locale.h>
-#endif
+#include "common.h"
+#include "plugin.h"
 
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libxfce4ui/libxfce4ui.h>
 
-#include "common.h"
-#include "plugin.h"
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 /*
  * Status Icon
  */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
-static MyPlugin*        status_icon_register            ();
-static gboolean         cb_status_icon_is_embedded      (GtkStatusIcon *status_icon);
-static void             cb_status_icon_activate         (MyPlugin *plugin);
-static void             cb_status_icon_popup_menu       (MyPlugin *plugin,
-                                                         guint button,
-                                                         guint activate_time);
-static void             cb_status_icon_quit             (MyPlugin *plugin);
-static void             cb_status_icon_finalize         (MyPlugin *plugin);
-static void             install_autostart_file          ();
-static void             update_autostart_file           ();
+static MyPlugin *
+status_icon_register (void);
+static void
+cb_status_icon_activate (MyPlugin *plugin);
+static void
+cb_status_icon_popup_menu (MyPlugin *plugin,
+                           guint button,
+                           guint activate_time);
+static void
+cb_status_icon_quit (MyPlugin *plugin);
+static void
+cb_status_icon_finalize (gpointer data,
+                         GObject *where_the_object_was);
+static void
+install_autostart_file (void);
+static void
+update_autostart_file (gboolean autostart);
 
 /*
  * Plugin Registration
@@ -58,41 +66,63 @@ main (gint argc,
 
   gtk_init (&argc, &argv);
 
-  g_set_application_name (_("Clipman"));
+  if (!WINDOWING_IS_X11 ())
+    {
+      g_message ("Systray icon is currently only supported on X11. Use the panel plugin instead.");
+      return EXIT_FAILURE;
+    }
+
   plugin = status_icon_register ();
+  if (plugin == NULL)
+    return EXIT_FAILURE;
+
   install_autostart_file ();
 
   gtk_main ();
 
   g_object_unref (plugin->status_icon);
 
-  return 0;
+  return EXIT_SUCCESS;
 }
 
 /*
  * Status Icon
  */
 
+static gboolean
+cb_status_icon_is_embedded (gpointer user_data)
+{
+  GtkStatusIcon *status_icon = user_data;
+
+  if (!gtk_status_icon_is_embedded (status_icon))
+    {
+      g_warning ("Status Icon is not embedded");
+      gtk_main_quit ();
+    }
+  return FALSE;
+}
+
 static MyPlugin *
 status_icon_register (void)
 {
   MyPlugin *plugin = plugin_register ();
-  GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
+  if (plugin == NULL)
+    return NULL;
 
   /* Menu Position Func */
-  plugin->menu_position_func = (GtkMenuPositionFunc)gtk_status_icon_position_menu;
+  plugin->menu_position_func = (GtkMenuPositionFunc) gtk_status_icon_position_menu;
 
   /* Status Icon */
-  if (gtk_icon_theme_has_icon (icon_theme, "clipman"))
+  if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), "clipman"))
     {
       plugin->status_icon = gtk_status_icon_new_from_icon_name ("clipman");
     }
   else
     {
-      plugin->status_icon = gtk_status_icon_new_from_stock (GTK_STOCK_PASTE);
+      plugin->status_icon = gtk_status_icon_new_from_icon_name ("edit-paste");
     }
-  gtk_status_icon_set_tooltip (plugin->status_icon, _("Clipman"));
-  g_timeout_add_seconds (60, (GSourceFunc)cb_status_icon_is_embedded, plugin->status_icon);
+  gtk_status_icon_set_tooltip_text (plugin->status_icon, _("Clipman"));
+  g_timeout_add_seconds (60, cb_status_icon_is_embedded, plugin->status_icon);
 
   /* Signals */
   g_signal_connect_swapped (plugin->status_icon, "activate",
@@ -100,22 +130,11 @@ status_icon_register (void)
   plugin->popup_menu_id =
     g_signal_connect_swapped (plugin->status_icon, "popup-menu",
                               G_CALLBACK (cb_status_icon_popup_menu), plugin);
-  g_object_weak_ref (G_OBJECT (plugin->status_icon), (GWeakNotify)cb_status_icon_finalize, plugin);
+  g_object_weak_ref (G_OBJECT (plugin->status_icon), cb_status_icon_finalize, plugin);
 
   return plugin;
 }
 
-static gboolean
-cb_status_icon_is_embedded (GtkStatusIcon *status_icon)
-{
-  if (!gtk_status_icon_is_embedded (status_icon))
-    {
-      g_warning ("Status Icon is not embedded");
-      gtk_main_quit ();
-    }
-  return FALSE;
-}
-
 static void
 cb_status_icon_activate (MyPlugin *plugin)
 {
@@ -149,8 +168,14 @@ cb_status_icon_popup_menu (MyPlugin *plugin, guint button, guint activate_time)
     }
 
   gtk_menu_set_screen (GTK_MENU (plugin->popup_menu), gtk_status_icon_get_screen (plugin->status_icon));
+
+  if (!gtk_widget_has_grab (plugin->popup_menu))
+    {
+      gtk_grab_add (plugin->popup_menu);
+    }
+
   gtk_menu_popup (GTK_MENU (plugin->popup_menu), NULL, NULL,
-                  (GtkMenuPositionFunc)gtk_status_icon_position_menu, plugin->status_icon,
+                  (GtkMenuPositionFunc) gtk_status_icon_position_menu, plugin->status_icon,
                   0, gtk_get_current_event_time ());
 }
 
@@ -163,8 +188,11 @@ cb_status_icon_quit (MyPlugin *plugin)
 }
 
 static void
-cb_status_icon_finalize (MyPlugin *plugin)
+cb_status_icon_finalize (gpointer data,
+                         GObject *where_the_object_was)
 {
+  MyPlugin *plugin = data;
+
   plugin_save (plugin);
   plugin_free (plugin);
 }
@@ -177,8 +205,8 @@ install_autostart_file (void)
   GKeyFile *keyfile;
   gchar *data;
 
-  sysfile = g_strdup (SYSCONFDIR"/xdg/autostart/"PACKAGE"-autostart.desktop");
-  userfile = g_strdup_printf ("%s/autostart/"PACKAGE"-autostart.desktop", g_get_user_config_dir ());
+  sysfile = g_strdup (SYSCONFDIR "/xdg/autostart/" PACKAGE "-autostart.desktop");
+  userfile = g_strdup_printf ("%s/autostart/" PACKAGE "-autostart.desktop", g_get_user_config_dir ());
 
   if (!g_file_test (sysfile, G_FILE_TEST_EXISTS))
     {
@@ -214,7 +242,7 @@ update_autostart_file (gboolean autostart)
   GKeyFile *keyfile;
   gchar *data;
 
-  userfile = g_strdup_printf ("%s/autostart/"PACKAGE"-autostart.desktop", g_get_user_config_dir ());
+  userfile = g_strdup_printf ("%s/autostart/" PACKAGE "-autostart.desktop", g_get_user_config_dir ());
 
   /* Check if the user autostart file exists */
   if (!g_file_test (userfile, G_FILE_TEST_EXISTS))
@@ -236,4 +264,4 @@ update_autostart_file (gboolean autostart)
   g_key_file_free (keyfile);
   g_free (userfile);
 }
-
+G_GNUC_END_IGNORE_DEPRECATIONS
